//
//  TagViewController.m

#import "TagViewController.h"
#import "TagAccessController.h"
#import "GlobalDefine.h"
#import "CustomCell.h"
#import "EpcConverter.h"
#import "BarcodeViewController.h"
#import "TagAccessController.h"
#import "NFCTableViewController.h"
#import "Utils.h"
#import "ModelSettings.h"
#import "RainTagHelper.h"
#import <AsReaderDockSDK/AsReaderDockSDK.h>
#import "GS1EPCRFIDSGTINData.h"

@interface TagViewController ()
{
    NSMutableArray *_accessoryList;
}

@property (nonatomic, strong, readwrite) TagAccessController *tagAccessController;
@end

@implementation TagViewController

#pragma mark - Lifecycle
//----------------------------------------------------------------
- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
}

#pragma mark - Private method
//----------------------------------------------------------------
- (void)updateData
{
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.tableView reloadData];
    });
}

- (NSString *)getEncodingString:(NSData *)pcEpc
{
    int encoding_type = (int)[[NSUserDefaults standardUserDefaults] integerForKey:@"RFIDEncoding"];
    return [EpcConverter toString:encoding_type data:pcEpc];
}


- (void)settingCellRFID:(CustomCell **)cell dictinonaryIndex:(NSInteger)index
{
    (*cell).tagEncoding.hidden = NO;
    (*cell).upcLabel.hidden = NO;
    (*cell).serialLabel.hidden = NO;
    NSDictionary *dic = [[ModelSettings sharedInstance].m_arReadTagData objectAtIndex:index];
    (*cell).tagEncoding.text = [RainTagHelper getPC: [dic objectForKey:df_CELL_TAG_RAW]];
    (*cell).tagHex.text = [RainTagHelper getEPC: [dic objectForKey:df_CELL_TAG_RAW]];
    NSMutableData *data = [dic objectForKey:df_CELL_TAG_RAW];
    if (data.length < 3) {
        (*cell).upcLabel.text = @"No UPC/JAN Decoded";
        (*cell).serialLabel.text = @"";
    } else {
        NSData *rangeData = [data subdataWithRange:NSMakeRange(2, data.length - 2)];
        GS1EPCRFIDSGTINData *epcData = [[GS1EPCRFIDSGTINData alloc] initWithEpcData:rangeData];
        if (epcData) {
            if (epcData.rfidProductCD == nil || [@"" isEqualToString:epcData.rfidProductCD]) {
                (*cell).upcLabel.text = @"No UPC/JAN Decoded";
            } else {
                (*cell).upcLabel.text = [NSString stringWithFormat:@"UPC/JAN:%@", epcData.rfidProductCD];
            }
            if (epcData.serialValue == nil || [@"" isEqualToString:epcData.serialValue]) {
                (*cell).serialLabel.text = @"";
            } else {
                (*cell).serialLabel.text = [NSString stringWithFormat:@"Serial:%@", epcData.serialValue];
            }
        } else {
            (*cell).upcLabel.text = @"No UPC/JAN Decoded";
            (*cell).serialLabel.text = @"";
        }
    }
    NSString *strRSSI = [dic objectForKey:df_CELL_TAG_RSSI];
    BOOL isRSSIOn = [[NSUserDefaults standardUserDefaults] boolForKey:@"RSSIOn"];
    
    if( isRSSIOn==YES && strRSSI.length>0 ){
        (*cell).tagRSSI.hidden = NO;
        (*cell).tagRSSI.text = [NSString stringWithFormat:@"RSSI : %@",strRSSI];
    }
    
    (*cell).imgType.image = [UIImage imageNamed:@"icon_rfid"];
}

- (void)settingCellNFC:(CustomCell **)cell dictinonaryIndex:(NSInteger)index
{
    //(*cell).tagEncoding.hidden = NO;
    NSDictionary *dic = [[ModelSettings sharedInstance].m_arReadTagData objectAtIndex:index];
    
    (*cell).tagEncoding.text = [NSString stringWithFormat:@" Encoding : %@",[self getEncodingString:[dic objectForKey:df_CELL_TAG_RAW]]];
    
    if ((*cell).tagHex.text.length == 24) {
        if ([(*cell).tagHex.text hasPrefix:@"010104"]) {
            (*cell).tagHex.textColor = [UIColor redColor];
        } else {
            (*cell).tagHex.textColor = [UIColor blackColor];
        }
    } else {
        (*cell).tagHex.textColor = [UIColor blackColor];
    }
}

- (void)settingCellRFIDLF:(CustomCell **)cell dictinonaryIndex:(NSInteger)index
{
    (*cell).tagEncoding.hidden = NO;
    NSDictionary *dic = [[ModelSettings sharedInstance].m_arReadTagData objectAtIndex:index];
    
    NSDictionary *getDic = [[LFTagParser sharedInstance] parseDataByProtocolType:[dic objectForKey:df_CELL_TAG_RAW] protocolType:ProtocolTypeLF_FDBX];
    NSString *natinalCode = [getDic valueForKey:Key_FDBX_NationalCode];
    NSString *countryCode = [getDic valueForKey:Key_FDBX_CountryCode];
    NSString *statusFlag = [getDic valueForKey:Key_FDBX_StatusFlag];
    NSString *animalIndicator = [getDic valueForKey:Key_FDBX_AnimalIndicator];
    NSString *reserved = [getDic valueForKey:Key_FDBX_Reserved];
    
    NSString *allData = [NSString stringWithFormat:@"%@%@%@%@%@", natinalCode, countryCode, statusFlag, animalIndicator, reserved];
    
    (*cell).tagEncoding.text = allData;
    (*cell).imgType.image = [UIImage imageNamed:@"icon_rfid"];
}

#pragma mark - UITableView datasource
//----------------------------------------------------------------
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if( [ModelSettings sharedInstance].m_arReadTagData==nil ){
        return 0;
    } else {
        return [[ModelSettings sharedInstance].m_arReadTagData count];
    }
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    CustomCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Cell" forIndexPath:indexPath];

    if( [[ModelSettings sharedInstance].m_arReadTagData count]==0 ) {
        return cell;
    }
    
    NSDictionary *dic = [[ModelSettings sharedInstance].m_arReadTagData objectAtIndex:[indexPath row]];
    cell.tagHex.text = [dic objectForKey:df_CELL_TAG_INFO];
    cell.tagCount.text = [dic objectForKey:df_CELL_TAG_COUNT];
    
    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    cell.tagRSSI.hidden = YES;
    cell.tagEncoding.hidden = YES;
    cell.upcLabel.hidden = YES;
    cell.serialLabel.hidden = YES;
    cell.tagHex.textColor = [UIColor blackColor];
    
    // Select icon
    ReceiveDataType dataType = [[dic objectForKey:df_CELL_DATA_TYPE] integerValue];
    switch (dataType) {
        case ReceiveDataTypeBarcode:
            cell.imgType.image = [UIImage imageNamed:@"icon_barcode"];
            break;
            
        case ReceiveDataTypeRFID:
            [self settingCellRFID:&cell dictinonaryIndex:[indexPath row]];
            break;
            
        case ReceiveDataTypeRFIDLF:
            [self settingCellRFIDLF:&cell dictinonaryIndex:[indexPath row]];
            break;
            
        case ReceiveDataTypeNFC:
            [self settingCellNFC:&cell dictinonaryIndex:[indexPath row]];
            cell.imgType.image = [UIImage imageNamed:@"icon_nfc"];
            break;
            
        default:
            cell.accessoryType = UITableViewCellAccessoryNone;
            break;
    }
    
    return cell;
}

#pragma mark - UITableView delegate
//----------------------------------------------------------------
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [ModelSettings sharedInstance].m_nSelectTagNum = (int)[indexPath row];
    NSDictionary *dic = [[ModelSettings sharedInstance].m_arReadTagData objectAtIndex:[indexPath row]];
    
    if( [[AsReaderInfo sharedInstance].deviceModelNumber containsString:@"L70D"]==YES ){
        NSData *data = [dic objectForKey:df_CELL_TAG_RAW];
   
        NSDictionary *dic = [[LFTagParser sharedInstance] parseDataByProtocolType:data protocolType:ProtocolTypeLF_FDBX];
        NSString *natinalCode = [dic valueForKey:Key_FDBX_NationalCode];
        NSString *countryCode = [dic valueForKey:Key_FDBX_CountryCode];
        NSString *statusFlag = [dic valueForKey:Key_FDBX_StatusFlag];
        NSString *animalIndicator = [dic valueForKey:Key_FDBX_AnimalIndicator];
        NSString *reserved = [dic valueForKey:Key_FDBX_Reserved];
        
        NSMutableString *allData = [NSMutableString string];
        [allData appendFormat:@"NationalCode:%@\n", natinalCode];
        [allData appendFormat:@"CountryCode:%@\n", countryCode];
        [allData appendFormat:@"StatusFlag:%@\n", statusFlag];
        [allData appendFormat:@"AnimalIndicator:%@\n", animalIndicator];
        [allData appendFormat:@"Reserved:%@\n", reserved];
        
        [Utils showAlert:@"TAG Info" message:allData view:self];
        return;
    }
    
    UIViewController *vc = nil;
    
    NSInteger deviceMode = [[dic objectForKey:df_CELL_TAG_DEVICE_MODE] integerValue];
    
    switch (deviceMode) {
        case ReaderModeRFID:
            vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"ID_TagAcessView"];
            break;
            
        case ReaderModeBarcode:
            vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"ID_BarcodeDetailView"];
            break;
            
        case ReaderModeNFC:{
            NSString *string = [dic objectForKey:df_CELL_TAG_INFO];
            if (string.length == 24 && [string hasPrefix:@"010104"] ) {
                vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"ID_NFCTableView"];
            }
        }
            break;
        default:
            break;
    }
    
    if( vc!=nil ){
        [self.navigationController pushViewController:vc animated:YES];
    }
}
@end
